//
//  ViewController.swift
//  8-3-3_EmitterKitSample
//
//  Created by oga on 2015/09/13.
//  Copyright © 2015年 Ryuji Watanabe. All rights reserved.
//

import UIKit
import EmitterKit

class ViewController: UIViewController {

    var sample2Listeners = [Listener]() // sample2で使用
    var sample5Listeners = [Listener]() // sample5で使用
    var sample6Listeners = [Listener]() // sample6で使用
    var sample7Listeners = [Listener]() // sample6で使用
    
    override func viewDidLoad() {
        super.viewDidLoad()
        // 一度のみ受け取る通知の例（Event）
        sample1()
        // 何度も受け取れる通知の例（Listener）
        sample2()
        // ターゲットを指定して通知
        sample3()
        // 値を必要としない通知例
        sample4()
        // NSNotificationを受信
        sample5()
        // UIViewのframeを監視
        sample6()
        // UIViewのframeを監視（新しい値のみ取得）
        sample7()
    }
    deinit {
        // Changeの場合、removeListenersを呼び出す必要がある
        self.view.removeListeners(sample6Listeners)
        self.view.removeListeners(sample7Listeners)
    }
    
    override func viewDidAppear(animated: Bool) {
        super.viewDidAppear(animated)
    }

    /**
    一度のみ受け取る通知の例（Event）
    */
    func sample1() {
        let counterPrint = Event<NSInteger>()
        
        counterPrint.once { counter in
            print("sample1 \(counter)")
        }
        
        counterPrint.emit(1)
        counterPrint.emit(2) // 1度しか呼ばれないためここは処理されない
    }
    /**
    何度も受け取れる通知の例（Listener）
    */
    func sample2() {
        let counterPrint = Event<NSInteger>()
        
        sample2Listeners += counterPrint.on { counter in
            print("sample2 \(counter)")
        }
        
        counterPrint.emit(1)
        counterPrint.emit(2)
    }
    /**
    ターゲットを指定して通知
    */
    func sample3() {
        let view = UIView()
        let counterPrint = Event<NSInteger>()
        
        counterPrint.once { counter in  // （１）
            print("sample3 \(counter)")
        }
        counterPrint.once(view) {       // （２）
            print("sample3 target \($0)")
        }
        counterPrint.emit(4) // （１）が呼ばれる
        counterPrint.emit(view, 2) // （２）が呼ばれる
    }
    /**
    値を必要としない通知例
    */
    func sample4() {
        let signal = Signal()
        signal.once {
            print("sample4")
        }
        signal.emit()
    }
    /**
    NSNotificationを受信
    */
    func sample5() {
        sample5Listeners += Notification(UIKeyboardWillShowNotification).on { info in
            print("sample5 \(info)")
        }
    }
    // UIViewのframeを監視
    func sample6() {
        sample6Listeners += self.view.on("frame") { (change: Change<NSValue>) in
            print("sample6 \(change.oldValue) -> \(change.newValue)")
        }
    }
    // UIViewのframeを監視（新しい値のみ取得）
    func sample7() {
        sample7Listeners += self.view.on("frame", .New) {
            (change: Change<NSValue>) in
            print("sample7 \(change.newValue)")
        }
    }
}

