// Playground - noun: a place where people can play

import UIKit

/////////////////////////////////////////
// code 3.30
/////////////////////////////////////////
class PersonClass {
    var name = String()
}
let person = PersonClass() // インスタンス作成
let person2 = person // person2 はperson の参照
person.name = "Swift"
print(person.name) // "Swift"
print(person2.name) // "Swift"

/////////////////////////////////////////
// code 3.31
/////////////////////////////////////////
struct PersonStruct {
    var name = String()
}
var personStruct = PersonStruct(name: "Swift")
let personStruct2 = personStruct
personStruct.name = "Jobs"
print(personStruct.name)    // "Jobs"
print(personStruct2.name)   // "Swift"

/////////////////////////////////////////
// code 3.34
/////////////////////////////////////////
// 構造体
struct FooStructure {
    var bar: Int
}
var foo = FooStructure(bar: 1)
foo.bar = 5

// クラス
class BarClass {
    var foo: Int
    init(foo : Int) { self.foo = foo }
}
let bar = BarClass(foo: 6)
bar.foo = 10

/////////////////////////////////////////
// code 3.35
/////////////////////////////////////////
struct FooStructure2 {
    var start: Int
    var end: Int
    var half: Int {
        get {
            return (end - start) / 2 + start
        }
        set {
            end = newValue * 2
        }
    }
    var multiplication: Int { // get、set の省略例
        return start * end
    }
}
var foo2 = FooStructure2(start: 10, end: 20)
print(foo2.half) // 15 (20-10)/2+10
foo2.half = 50
print(foo2.half) // 55 (100-10)/2+10
print(foo2.multiplication) // 1000

/////////////////////////////////////////
// code 3.36
/////////////////////////////////////////
struct SomeStructure {
    static var typeProperty = 1
    static var typeProperty2: String {
        return "typeProperty2"
    }
}
class SomeClass {
    static var typeProperty = 1
    static var typeProperty2: String {
        return "typeProperty2"
    }
}
SomeStructure.typeProperty = 15
SomeClass.typeProperty = 20
print(SomeStructure.typeProperty) // 15
print(SomeStructure.typeProperty2) // "typeProperty2"
print(SomeClass.typeProperty) // 20
print(SomeClass.typeProperty2) // "typeProperty2"

/////////////////////////////////////////
// code 3.37
/////////////////////////////////////////
// クラスの場合
class PetClass {
    lazy var name: String = "Yukichi"
}
let pet = PetClass()    // この時点でpet.nameは入っていない
print(pet.name)       // "Yukichi" ここで初めてデフォルト値がセットされる
// 構造体の場合
struct PetStructure {
    lazy var name: String = "Yukichi"
}
var petStructure = PetStructure()   // petStructure.nameはnil
print(petStructure.name)          // "Yukichi"

/////////////////////////////////////////
// code 3.38
/////////////////////////////////////////
class WaitClass {
    init() {
        sleep(3)    // 3秒待つ
    }
    func description() -> String {
        return "WaitClass description"
    }
}
class LazyClass {
    lazy var waitClass = WaitClass()
}
let lazyClass = LazyClass() // この処理はすぐ終わる

lazyClass.waitClass                         // waitClassにアクセス
print(lazyClass.waitClass.description())  // 表示されるのに3秒かかります

lazyClass.waitClass                         // 2度目のアクセス
print(lazyClass.waitClass.description())  // 瞬時に表示

/////////////////////////////////////////
// code 3.39
/////////////////////////////////////////
class PetClass2 {
    var name: String {
        willSet {
            print("willSet old name = \(name)")
            print("willSet new name = \(newValue)")
        }
        didSet {
            print("didSet old name = \(oldValue)")
            name = "\(name) is Best"
        }
    }
    init() {
        name = "Yukichi"
    }
}
var pet2 = PetClass2()
pet2.name = "Kamekichi"
// "willSet new name = Kamekichi"
// "didSet old name = Yukichi"

print(pet2.name)
// "Kamekichi is Best"

/////////////////////////////////////////
// code 3.40
/////////////////////////////////////////
class Counter {
    var count = 0
    func increment() {
        count++
    }
    func incrementBy(amount: Int, numberOfTimes: Int) {
        count += amount * numberOfTimes
    }
    func reset() {
        count = 0
    }
    func resetBy(count count: Int) {
        self.count = count
    }
}
let counter = Counter()
counter.increment()                         // counter.count = 1
counter.incrementBy(12, numberOfTimes: 3)   // counter.count = 37
counter.reset()                             // counter.count = 0
counter.resetBy(count: 10)                  // counter.count = 10

/////////////////////////////////////////
// code 3.41
/////////////////////////////////////////
struct CounterStructure {
    var count = 0
    mutating func increment() {
        count++
    }
    mutating func incrementBy(amount: Int, numbrtOfTimes: Int) {
        count += amount * numbrtOfTimes
    }
    mutating func resetBy(count count: Int) {
        self.count = count
    }
}

var counterStruct = CounterStructure()
counterStruct.increment()                       // counterStruct.count = 1
counterStruct.incrementBy(20, numbrtOfTimes: 5) // counterStruct.count = 101
counterStruct.resetBy(count: 100)               // counterStruct.count = 100

/////////////////////////////////////////
// code 3.42
/////////////////////////////////////////
struct Point {
    var x = 0.0, y = 0.0
    mutating func moveBy(x x: Double, y: Double) {
        self = Point(x: self.x + x, y: self.y + y)
    }
}

var point = Point()                     // x=0.0 y=0.0
point.moveBy(x: 10.5, y: 20.6)
print("x=\(point.x) y=\(point.y)")    // "x=10.5 y=20.6"

/////////////////////////////////////////
// code 3.43
/////////////////////////////////////////
class SomeClass2 {
    static func staticSomeTypeMethod() {
        print("staticSomeTypeMethod")
    }
    class func classSomeTypeMethod() {
        print("classSomeTypeMethod")
    }
}
struct SomeStructure2 {
    static func staticSomeTypeMethod() {
        print("staticSomeTypeMethod")
    }
}
SomeClass2.staticSomeTypeMethod()
SomeClass2.classSomeTypeMethod()
SomeStructure2.staticSomeTypeMethod()

/////////////////////////////////////////
// code 3.44
/////////////////////////////////////////
class SomeClass3 {
    class func someTypeMethod() {
        print("someTypeMethod")
    }
    func someMethod() {
        self.dynamicType.someTypeMethod()
    }
}
let someClass3 = SomeClass3()
someClass3.someMethod()
someClass3.dynamicType.someTypeMethod()

/////////////////////////////////////////
// code 3.46
/////////////////////////////////////////
struct CalcStructure {
    let baseInt: Int
    subscript(index: Int) -> Int {
        return baseInt + index
    }
}
let calcStructure = CalcStructure(baseInt: 10)
print(calcStructure[5])

/////////////////////////////////////////
// code 3.47
/////////////////////////////////////////
class CalcClass {
    var baseInt: Int
    init(baseInt: Int) {
    self.baseInt = baseInt
    }
    subscript(index: Int) -> Int {
        get {
            return self.baseInt + index
        }
        set {
            baseInt += index * newValue
        }
    }
}
let calcClass = CalcClass(baseInt: 10)
calcClass[5] = 10
print(calcClass.baseInt)  // 60
print(calcClass[196])     // 256

/////////////////////////////////////////
// code 3.49
/////////////////////////////////////////
class CalcClass2: CustomStringConvertible {
    var baseInt: Int
    init(baseInt: Int) {
        self.baseInt = baseInt
    }
    func append(index: Int) -> Int {
        return baseInt + index
    }
    var description: String {
        return "super class \(baseInt)"
    }
}
class SubCalcClass2: CalcClass2 {
    override var description: String {
        return "sub class \(baseInt)"
    }
}
// スーパークラスでメソッド・プロパティを呼ぶ
let superClass = CalcClass2(baseInt: 5)
let superAppend = superClass.append(10)
let superString = superClass.description
print(superAppend) // 15
print(superString) // "super class 5"

// サブクラスでメソッド・プロパティを呼ぶ
let subClass = SubCalcClass2(baseInt: 8)
let subAppend = subClass.append(2)
let subString = subClass.description
print(subAppend) // 10
print(subString) // "sub class 8"

/////////////////////////////////////////
// code 3.52
/////////////////////////////////////////
class SimpleClass {
    var name: String
    init() {
        name = "Yukichi"
    }
}
struct SimpleStruct {
    var name: String
    init() {
        name = "Yukichi"
    }
}
let simpleClass = SimpleClass()
print(simpleClass.name)   // "Yukichi"
var simpleStruct = SimpleStruct()
print(simpleStruct.name)  // "Yukichi"

/////////////////////////////////////////
// code 3.53
/////////////////////////////////////////
struct SimpleStruct2 {
    var name: String
}
let simpleClass2 = SimpleStruct2(name: "Yukichi") // nameという引数のinit
print(simpleClass2.name)  // "Yukichi"

/////////////////////////////////////////
// code 3.54
/////////////////////////////////////////
class SimpleClass3 {
    var name = "Yukichi"
    var type: String?
}
struct SimpleStruct3 {
    var name = "Yukichi"
}
struct OptionalStruct {
    var name = "Yukichi"
    var type: String?
}
let simpleClass3 = SimpleClass3()
print(simpleClass3.name)      // "Yukichi"
var simpleStruct3 = SimpleStruct3()
print(simpleStruct3.name)     // "Yukichi"
var optionalStruct = OptionalStruct(name: "Jobs", type: "Human")
print(optionalStruct.name)    // "Jobs"

/////////////////////////////////////////
// code 3.55
/////////////////////////////////////////
class PersonClass3 {
    let names:[String] = {
        var array:[String] = Array()
        for i in 0...3 {
            array.append("Yukichi_\(i)")
        }
        return array
    }()
}
let person3 = PersonClass3()
for name in person3.names {
    print(name)
}
// "Yukichi_0"
// "Yukichi_1"
// "Yukichi_2"
// "Yukichi_3"

/////////////////////////////////////////
// code 3.56
/////////////////////////////////////////
class PersonClass4 {
    let name: String
    let country = "Japan"
    init() {
        self.name = "Kamekichi"
    }
}
let person4 = PersonClass4()
print(person4.name)    // "Kamekichi"
print(person4.country) // "Japan"

/////////////////////////////////////////
// code 3.57
/////////////////////////////////////////
struct MultipleInitStruct: CustomStringConvertible {
    var name = "Yukichi"
    var type = "Dog"
    init() {}
    init(name: String, type: String) {
        self.name = name
        self.type = type
    }
    init(humanName: String) {
        self.init(name: humanName, type: "Human")
    }
    var description: String {
        return "name=\(name), type=\(type)"
    }
}
var sample = MultipleInitStruct(humanName: "Jobs")
print(sample.description) // "name=Jobs, type=Human"


/////////////////////////////////////////
// code 3.58
/////////////////////////////////////////
class MultipleInitClass: CustomStringConvertible {
    var name = "Yukichi"
    var type = "Dog"
    init() {}
    init(name: String, type: String) {
        self.name = name
        self.type = type
    }
    convenience init(humanName: String) {
        self.init(name: humanName, type: "Human")
    }
    var description: String {
        return "name=\(name), type=\(type)"
    }
}
var multipleInit = MultipleInitClass(humanName: "Jobs")
print(multipleInit.description) // "name=Jobs, type=Human"

/////////////////////////////////////////
// code 3.59
/////////////////////////////////////////
class ParameterClass {
    var code: Int
    var name: String
    init(code paramCode: Int, name paramName: String) {
        code = paramCode
        name = paramName
    }
}
let paramClass = ParameterClass(code: 3, name: "Yukichi")

/////////////////////////////////////////
// code 3.60
/////////////////////////////////////////
class PositionClass {
    var x: Double, y: Double, z: Double
    init(_ paramX: Double, _ paramY: Double, _ paramZ: Double) {
        x = paramX; y = paramY; z = paramZ
    }
}
let position = PositionClass(10.0, 20.0, 1.0)

/////////////////////////////////////////
// code 3.61
/////////////////////////////////////////
class ElectricFan {
    var maxWindLevel = 0
    var description: String {
        return "maxWindLevel = \(maxWindLevel)"
    }
}
let electricFan = ElectricFan()
print("electricFan: \(electricFan.description)")
// "electricFan: maxWindLevel = 0"
    
class PowerfulFan: ElectricFan {
    override init() {
        super.init()
        maxWindLevel = 5
    }
}
let powerfulFan = PowerfulFan()
print("powerfulFan: \(powerfulFan.description)")
// "powerfulFan: maxWindLevel = 5"

/////////////////////////////////////////
// code 3.62
/////////////////////////////////////////
class ElectricFan2 {
    var maxWindLevel: Int
    init(maxWindLevel: Int) {
        self.maxWindLevel = maxWindLevel
    }
    convenience init() {
        self.init(maxWindLevel:0)
    }
    var description: String {
        return "maxWindLevel = \(maxWindLevel)"
    }
}
let electricFan2 = ElectricFan2()
print("electricFan2: \(electricFan2.description)")
// "electricFan2: maxWindLevel = 0"

class PowerfulFan2: ElectricFan2 {
    var maker: String
    override init(maxWindLevel: Int) {
        maker = "MYNAVI"
        super.init(maxWindLevel: maxWindLevel)
    }
}
let powerfulFan2 = PowerfulFan2()
print("powerfulFan2: \(powerfulFan2.description)")
// "powerfulFan: maxWindLevel = 0"

/////////////////////////////////////////
// code 3.64
/////////////////////////////////////////
class SomeClass4 {
    var name = ""
    init?(newName: String?) {
        guard let value = newName else {
            return nil
        }
        self.name = value
    }
}
let foo3 = SomeClass4(newName: nil)      // nil
let bar3 = SomeClass4(newName: "Good!")  // SomeClass4?型

/////////////////////////////////////////
// code 3.66
/////////////////////////////////////////
public class PublicClass {
    var property1 = 1                 // public
    func publicMethod() {}            // public
    private func privateMethod() {}   // private
    internal func internalMethod() {} // internal
}
internal class InternalClass {
    var property1 = 1                 // internal
    func publicMethod() {}            // internal
//    public func publicMethod() {}     // エラー
    private func privateMethod() {}   // private
    internal func internalMethod() {} // internal
}
private class PrivateClass {
    var property1 = 1                 // private
    func publicMethod() {}            // private
//    public func publicMethod() {}     // エラー
//    internal func internalMethod() {} // エラー
}

/////////////////////////////////////////
// code 3.69
/////////////////////////////////////////
public class Sample {
    private var value1 = 0               // get、setともprivate
    private(set) var value2 = 0          // getはpublic、setはprivate
    internal private(set) var value3 = 0 // getはinternal、setはprivate
//    private internal(set) var value4 = 0 // エラー
}

/////////////////////////////////////////
// code 3.72, 3.73, 3.74
/////////////////////////////////////////
enum Dog {
    case Pug
    case FrenchBulldog
    case BostonTerrier
}
enum Authority {
        case Admin, Technical, Sales, Finance
}

var category = Dog.FrenchBulldog
category = .Pug
category = .BostonTerrier
switch category {
case .FrenchBulldog:
    print("furebull")
case .BostonTerrier:
    print("boss teri")
case .Pug:
    print("pug")
}
// "boss teri"

/////////////////////////////////////////
// code 3.75
/////////////////////////////////////////
let auth = Authority.Sales
switch auth {
case .Sales:
    print("authority is sales")
default:
        print("authority is not sales")
}
// "authority is sales"


/////////////////////////////////////////
// code 3.76
/////////////////////////////////////////
enum Result {
    case Error(Int, String)
    case Status(Int)
}
var result = Result.Error(-1, "error!")
switch result {
case .Error(let code, let message):
        print("ERROR > code:\(code) message:\(message)")
case .Status(let code):
        print("Success > code:\(code)")
}

/////////////////////////////////////////
// code 3.77
/////////////////////////////////////////
enum MoveKey: String {
    case Up    = "w"
    case Left  = "d"
    case Down  = "x"
    case Right = "a"
}
var moveKey = MoveKey.Left
print(moveKey.rawValue) // "d"

/////////////////////////////////////////
// code 3.78,3.79
/////////////////////////////////////////
enum DogCategory: UInt {
    case Pug = 1, FrenchBulldog, BostonTerrier
}
var dogCategory = DogCategory.FrenchBulldog
print(dogCategory.rawValue) // 2

if let dog = DogCategory(rawValue: 3) {
    switch dog {
case .Pug:
            print("pug")
case .FrenchBulldog:
            print("frenchBulldog")
case .BostonTerrier:
    print("bostonTerrier")
    }
}
    else {
    print("no category")
}
// "bostonTerrier"



