//: Playground - noun: a place where people can play

import UIKit

///////////////////////////////////
//
// 4-4 Optional Chaining
//
///////////////////////////////////

// 4-4-2 Optional Chainingの利用

class Teacher {
    var classroom:Classroom?
}

class Classroom {
    var numberOfStudents = 30
}

let teacher = Teacher()
// アンラップするとアボートする
//let studentsCount = teacher.classroom!.numberOfStudents // Abort
// println("先生は \(studentsCount) 人のクラスを担任している。")

// Optional Bindingを用いる
if let classroom = teacher.classroom {
    let studentsCount = classroom.numberOfStudents
    print("先生は \(studentsCount) 人のクラスを担任している。")
} else {
    print("先生は担任していない。")
}

// Optional Chaining
let studentsCount = teacher.classroom?.numberOfStudents
print("先生は \(studentsCount) 人のクラスを担任している。")

// Optional ChainingとOptional Bindingの組み合わせ
teacher.classroom = Classroom()
if let studentsCount = teacher.classroom?.numberOfStudents {
    print("先生は \(studentsCount) 人のクラスを担任している。")
} else {
    print("先生は担任していない。")
}


// 4-4-3 Optional Chainingを用いた代入操作

// 上書きだけ実行される
var someInt:Int? = 10
someInt? = 20 // 20
var nilInt:Int? = nil
nilInt? = 20 // nil



// numberOfStudensプロパティはInt型なので、Int?型の代入はできない
teacher.classroom?.numberOfStudents = someInt ?? 0



// 代入時にj++は実行される
var i:Int? = nil
var j:Int  = 0
i? = j++
print(j) // 1


// 4-4-4 VoidとOptional Chaining

// 定義を見るにはVoidをcmd + click
let void:Void


// Void型でもnilチェックが可能
extension Classroom {
    func printCount() -> Void {
        print("生徒数: \(numberOfStudents)")
    }
}

teacher.classroom = nil
if teacher.classroom?.printCount() != nil {
    print("printCount()を実行")
} else {
    print("printCount()を無視")
}


// 4-4-5 プロトコルの任意実装なメソッドの実行

@objc protocol MyDelegate {
    optional func optionalMethodForMe(object:MyObject)
}

class MyObject:NSObject {
    weak var delegate:MyDelegate?
    
    func doDelegateMethod() {
        /*
        Objective-Cでは
        if ([self.delegate respondsToSelector:@selector(optionalMethodForMe:)]) {
        [self.delegate optionalMethodForMe:self];
        }
        のように書いていたコードがSwiftでは次のように１行で書ける
        */
        self.delegate?.optionalMethodForMe?(self)
    }
}

class NotImplementObject: NSObject, MyDelegate {
}

class ImplementObject: NSObject, MyDelegate {
    func optionalMethodForMe(object: MyObject) {
        print("do it")
    }
}

let myObj = MyObject()

// 実行する
let impletemted = ImplementObject()
myObj.delegate = impletemted
myObj.doDelegateMethod()

// 実行しないがクラッシュもしない
let notImpletemted = NotImplementObject()
myObj.delegate = notImpletemted
myObj.doDelegateMethod()


///////////////////////////////////
//
// コラム: nilはどんな値よりも小さいと判断される
//
///////////////////////////////////

nil < 0
nilInt < Int.min
nil < -Double.infinity

var str = "string"

if Int(str) < 10 {
    print("実行される")
}

