// コード8.57 Factory Method パターンの例

import UIKit

protocol ApiProtocol {
    func detail() -> String
}
// 本番クラス
class ApiClass: ApiProtocol {
    var url:String
    init() {
        url = "http://live.localhost/"
    }
    func detail() -> String {
        let result = url
        // サーバとのやりとりで結果を返す
        // この例ではurl を返します。
        return result
    }
}
// テストクラス
class TestApiClass: ApiClass {
    override init () {
        super.init()
        self.url = "http://test.localhost/"
    }
}
// スタブ用クラス
class StubApiClass: ApiProtocol {
    func detail() -> String {
        return " スタブデータです"
    }
}
// API カテゴリー
enum ApiCategory {
    case Live, Test, Stub
}
// Factory クラス
class ApiFactory {
    // Api のカテゴリで返すインスタンスを決める
    class func apiClass(category:ApiCategory) -> ApiProtocol {
        switch category {
        case .Live:
            return ApiClass()
        case .Test:
            return TestApiClass()
        case .Stub:
            return StubApiClass()
        }
    }
}
// 実際に使う
let liveApi = ApiFactory.apiClass(.Live)
let testApi = ApiFactory.apiClass(.Test)
let stubApi = ApiFactory.apiClass(.Stub)
print(liveApi.detail()) // "http://live.localhost/"
print(testApi.detail()) // "http://test.localhost/"
print(stubApi.detail()) // " スタブデータです"
